@@ -1,6 +1,7 @@
 import html
 import json
 import string
+import typing
 import warnings
 from urllib.parse import urlparse
 
@@ -35,7 +36,10 @@ from allauth import app_settings as allauth_app_settings
 from allauth.account import app_settings, signals
 from allauth.core import context, ratelimit
 from allauth.core.internal.adapter import BaseAdapter
-from allauth.core.internal.httpkit import headed_redirect_response
+from allauth.core.internal.httpkit import (
+    headed_redirect_response,
+    is_headless_request,
+)
 from allauth.utils import generate_unique_username, import_attribute
 
 
@@ -58,17 +62,22 @@ class DefaultAccountAdapter(BaseAdapter):
         "email_password_mismatch": _(
             "The email address and/or password you specified are not correct."
         ),
+        "phone_password_mismatch": _(
+            "The phone number and/or password you specified are not correct."
+        ),
         "email_taken": _("A user is already registered with this email address."),
         "enter_current_password": _("Please type your current password."),
         "incorrect_code": _("Incorrect code."),
         "incorrect_password": _("Incorrect password."),
         "invalid_or_expired_key": _("Invalid or expired key."),
+        "invalid_login": _("Invalid login."),
         "invalid_password_reset": _("The password reset token was invalid."),
         "max_email_addresses": _("You cannot add more than %d email addresses."),
         "too_many_login_attempts": _(
             "Too many failed login attempts. Try again later."
         ),
-        "unknown_email": _("The email address is not assigned to any user account"),
+        "unknown_email": _("The email address is not assigned to any user account."),
+        "unknown_phone": _("The phone number is not assigned to any user account."),
         "unverified_primary_email": _("Your primary email address must be verified."),
         "username_blacklisted": _(
             "Username can not be used. Please use other username."
@@ -79,6 +88,8 @@ class DefaultAccountAdapter(BaseAdapter):
         "username_taken": AbstractUser._meta.get_field("username").error_messages[
             "unique"
         ],
+        "select_only_one": _("Please select only one."),
+        "same_as_current": _("The new value must be different from the current one."),
     }
 
     def stash_verified_email(self, request, email):
@@ -112,10 +123,7 @@ class DefaultAccountAdapter(BaseAdapter):
             .exclude(pk=email_address.pk)
             .exists()
         )
-        login_by_email = (
-            app_settings.AUTHENTICATION_METHOD
-            == app_settings.AuthenticationMethod.EMAIL
-        )
+        login_by_email = app_settings.LOGIN_METHODS == {app_settings.LoginMethod.EMAIL}
         if email_address.primary:
             if has_other:
                 # Don't allow, let the user mark one of the others as primary
@@ -191,16 +199,21 @@ class DefaultAccountAdapter(BaseAdapter):
             msg.content_subtype = "html"  # Main content is now text/html
         return msg
 
-    def send_mail(self, template_prefix, email, context):
+    def send_mail(self, template_prefix: str, email: str, context: dict) -> None:
+        request = globals()["context"].request
         ctx = {
+            "request": request,
             "email": email,
-            "current_site": get_current_site(globals()["context"].request),
+            "current_site": get_current_site(request),
         }
         ctx.update(context)
         msg = self.render_mail(template_prefix, email, ctx)
         msg.send()
 
     def get_signup_redirect_url(self, request):
+        """
+        Returns the default URL to redirect to directly after signing up.
+        """
         return resolve_url(app_settings.SIGNUP_REDIRECT_URL)
 
     def get_login_redirect_url(self, request):
@@ -349,7 +362,7 @@ class DefaultAccountAdapter(BaseAdapter):
                 raise self.validation_error("username_taken")
         return username
 
-    def clean_email(self, email):
+    def clean_email(self, email: str) -> str:
         """
         Validates an email value. You can hook into this if you want to
         (dynamically) restrict what email addresses can be chosen.
@@ -367,6 +380,13 @@ class DefaultAccountAdapter(BaseAdapter):
         validate_password(password, user)
         return password
 
+    def clean_phone(self, phone: str) -> str:
+        """
+        Validates a phone number. You can hook into this if you want to
+        (dynamically) restrict what phone numbers can be chosen.
+        """
+        return phone
+
     def validate_unique_email(self, email):
         return email
 
@@ -383,7 +403,7 @@ class DefaultAccountAdapter(BaseAdapter):
         Wrapper of `django.contrib.messages.add_message`, that reads
         the message text from a template.
         """
-        if getattr(getattr(request, "allauth", None), "headless", None):
+        if is_headless_request(request):
             return
         if "django.contrib.messages" in settings.INSTALLED_APPS:
             if message:
@@ -477,9 +497,14 @@ class DefaultAccountAdapter(BaseAdapter):
     ):
         from .utils import get_login_redirect_url
 
-        response = HttpResponseRedirect(
-            get_login_redirect_url(request, redirect_url, signup=signup)
-        )
+        if is_headless_request(request):
+            from allauth.headless.base.response import AuthenticationResponse
+
+            response = AuthenticationResponse(request)
+        else:
+            response = HttpResponseRedirect(
+                get_login_redirect_url(request, redirect_url, signup=signup)
+            )
 
         if signal_kwargs is None:
             signal_kwargs = {}
@@ -601,7 +626,7 @@ class DefaultAccountAdapter(BaseAdapter):
     def should_send_confirmation_mail(self, request, email_address, signup) -> bool:
         return True
 
-    def send_account_already_exists_mail(self, email):
+    def send_account_already_exists_mail(self, email: str) -> None:
         from allauth.account.internal import flows
 
         signup_url = flows.signup.get_signup_url(context.request)
@@ -609,7 +634,6 @@ class DefaultAccountAdapter(BaseAdapter):
             context.request
         )
         ctx = {
-            "request": context.request,
             "signup_url": signup_url,
             "password_reset_url": password_reset_url,
         }
@@ -689,6 +713,10 @@ class DefaultAccountAdapter(BaseAdapter):
         email = EmailAddress.objects.get_primary_email(user)
         if email:
             credentials["email"] = email
+        if app_settings.LoginMethod.PHONE in app_settings.LOGIN_METHODS:
+            phone_verified = self.get_phone(user)
+            if phone_verified:
+                credentials["phone"] = phone_verified[0]
         reauth_user = self.authenticate(context.request, **credentials)
         return reauth_user is not None and reauth_user.pk == user.pk
 
@@ -719,11 +747,14 @@ class DefaultAccountAdapter(BaseAdapter):
     def get_login_stages(self):
         ret = []
         ret.append("allauth.account.stages.LoginByCodeStage")
+        ret.append("allauth.account.stages.PhoneVerificationStage")
         ret.append("allauth.account.stages.EmailVerificationStage")
         if allauth_app_settings.MFA_ENABLED:
             from allauth.mfa import app_settings as mfa_settings
 
             ret.append("allauth.mfa.stages.AuthenticateStage")
+            if mfa_settings.TRUST_ENABLED:
+                ret.append("allauth.mfa.stages.TrustStage")
 
             if mfa_settings.PASSKEY_SIGNUP_ENABLED:
                 ret.append("allauth.mfa.webauthn.stages.PasskeySignupStage")
@@ -788,18 +819,30 @@ class DefaultAccountAdapter(BaseAdapter):
         """
         return self._generate_code()
 
+    def generate_password_reset_code(self) -> str:
+        """
+        Generates a new password reset code.
+        """
+        return self._generate_code(length=8)
+
     def generate_email_verification_code(self) -> str:
         """
         Generates a new email verification code.
         """
         return self._generate_code()
 
-    def _generate_code(self):
+    def generate_phone_verification_code(self) -> str:
+        """
+        Generates a new phone verification code.
+        """
+        return self._generate_code()
+
+    def _generate_code(self, length=6):
         forbidden_chars = "0OI18B2ZAEU"
         allowed_chars = string.ascii_uppercase + string.digits
         for ch in forbidden_chars:
             allowed_chars = allowed_chars.replace(ch, "")
-        return get_random_string(length=6, allowed_chars=allowed_chars)
+        return get_random_string(length=length, allowed_chars=allowed_chars)
 
     def is_login_by_code_required(self, login) -> bool:
         """
@@ -821,6 +864,57 @@ class DefaultAccountAdapter(BaseAdapter):
             return False
         return method is None or method in value
 
+    def phone_form_field(self, **kwargs):
+        """
+        Returns a form field used to input phone numbers.
+        """
+        from allauth.account.fields import PhoneField
 
-def get_adapter(request=None):
+        return PhoneField(**kwargs)
+
+    def send_unknown_account_sms(self, phone: str, **kwargs):
+        """
+        In case enumeration prevention is enabled, and, a verification code
+        is requested for an unlisted phone number, this method is invoked to
+        send a text explaining that no account is on file.
+        """
+        pass
+
+    def send_verification_code_sms(self, user, phone: str, code: str, **kwargs):
+        """
+        Sends a verification code.
+        """
+        raise NotImplementedError
+
+    def set_phone(self, user, phone: str, verified: bool):
+        """
+        Sets the phone number (and verified status) for the given user.
+        """
+        raise NotImplementedError
+
+    def get_phone(self, user) -> typing.Optional[typing.Tuple[str, bool]]:
+        """
+        Returns the phone number stored for the given user. A tuple of the
+        phone number itself, and whether or not the phone number was verified is
+        returned.
+        """
+        raise NotImplementedError
+
+    def set_phone_verified(self, user, phone: str):
+        """
+        Marks the specified phone number for the given user as
+        verified. Note that the user is already expected to have
+        the phone number attached to the account.
+        """
+        raise NotImplementedError
+
+    def get_user_by_phone(self, phone: str):
+        """
+        Looks up a user given the specified phone number. Returns ``None`` if no user
+        was phone.
+        """
+        raise NotImplementedError
+
+
+def get_adapter(request=None) -> DefaultAccountAdapter:
     return import_attribute(app_settings.ADAPTER)(request)
