@@ -1,5 +1,6 @@
+import warnings
 from enum import Enum
-from typing import Set, Union
+from typing import FrozenSet, Set, Union
 
 
 class AppSettings:
@@ -8,6 +9,11 @@ class AppSettings:
         EMAIL = "email"
         USERNAME_EMAIL = "username_email"
 
+    class LoginMethod(str, Enum):
+        USERNAME = "username"
+        EMAIL = "email"
+        PHONE = "phone"
+
     class EmailVerificationMethod(str, Enum):
         # After signing up, keep the user account inactive until the email
         # address is verified
@@ -72,7 +78,12 @@ class AppSettings:
         """
         The user is required to hand over an email address when signing up
         """
-        return self._setting("EMAIL_REQUIRED", False)
+        warnings.warn(
+            "app_settings.EMAIL_REQUIRED is deprecated, use: app_settings.SIGNUP_FIELDS['email']['required']",
+            stacklevel=3,
+        )
+        email = self.SIGNUP_FIELDS.get("email")
+        return email and email["required"]
 
     @property
     def EMAIL_VERIFICATION(self):
@@ -109,8 +120,32 @@ class AppSettings:
 
     @property
     def AUTHENTICATION_METHOD(self):
-        ret = self._setting("AUTHENTICATION_METHOD", self.AuthenticationMethod.USERNAME)
-        return self.AuthenticationMethod(ret)
+        warnings.warn(
+            "app_settings.AUTHENTICATION_METHOD is deprecated, use: app_settings.LOGIN_METHODS",
+            stacklevel=3,
+        )
+        methods = self.LOGIN_METHODS
+        if self.LoginMethod.EMAIL in methods and self.LoginMethod.USERNAME in methods:
+            return "username_email"
+        elif self.LoginMethod.EMAIL in methods:
+            return "email"
+        elif self.LoginMethod.USERNAME in methods:
+            return "username"
+        else:
+            raise NotADirectoryError
+
+    @property
+    def LOGIN_METHODS(self) -> FrozenSet[LoginMethod]:
+        methods = self._setting("LOGIN_METHODS", None)
+        if methods is None:
+            auth_method = self._setting(
+                "AUTHENTICATION_METHOD", self.AuthenticationMethod.USERNAME
+            )
+            if auth_method == self.AuthenticationMethod.USERNAME_EMAIL:
+                methods = {self.LoginMethod.EMAIL, self.LoginMethod.USERNAME}
+            else:
+                methods = {self.LoginMethod(auth_method)}
+        return frozenset([self.LoginMethod(m) for m in methods])
 
     @property
     def EMAIL_MAX_LENGTH(self):
@@ -119,6 +154,18 @@ class AppSettings:
         """
         return self._setting("EMAIL_MAX_LENGTH", 254)
 
+    @property
+    def PHONE_VERIFICATION_ENABLED(self):
+        return self._setting("PHONE_VERIFICATION_ENABLED", True)
+
+    @property
+    def PHONE_VERIFICATION_MAX_ATTEMPTS(self):
+        return self._setting("PHONE_VERIFICATION_MAX_ATTEMPTS", 3)
+
+    @property
+    def PHONE_VERIFICATION_TIMEOUT(self):
+        return self._setting("PHONE_VERIFICATION_TIMEOUT", 15 * 60)
+
     @property
     def UNIQUE_EMAIL(self):
         """
@@ -131,15 +178,22 @@ class AppSettings:
         """
         Signup email verification
         """
-        return self._setting("SIGNUP_EMAIL_ENTER_TWICE", False)
+        warnings.warn(
+            "app_settings.SIGNUP_EMAIL_ENTER_TWICE is deprecated, use: 'email2' in app_settings.SIGNUP_FIELDS",
+            stacklevel=3,
+        )
+        return "email2" in self.SIGNUP_FIELDS
 
     @property
     def SIGNUP_PASSWORD_ENTER_TWICE(self):
         """
         Signup password verification
         """
-        legacy = self._setting("SIGNUP_PASSWORD_VERIFICATION", True)
-        return self._setting("SIGNUP_PASSWORD_ENTER_TWICE", legacy)
+        warnings.warn(
+            "app_settings.SIGNUP_PASSWORD_ENTER_TWICE is deprecated, use: 'password2' in app_settings.SIGNUP_FIELDS",
+            stacklevel=3,
+        )
+        return "password2" in self.SIGNUP_FIELDS
 
     @property
     def SIGNUP_REDIRECT_URL(self):
@@ -180,6 +234,8 @@ class AppSettings:
         ret = {
             # Change password view (for users already logged in)
             "change_password": "5/m/user",
+            # Change phone number
+            "change_phone": "1/m/user",
             # Email management (e.g. add, remove, change primary)
             "manage_email": "10/m/user",
             # Request a password reset, global rate limit per IP
@@ -223,12 +279,46 @@ class AppSettings:
         """
         return self._setting("SIGNUP_FORM_HONEYPOT_FIELD", None)
 
+    @property
+    def SIGNUP_FIELDS(self) -> dict:
+        fields = self._setting("SIGNUP_FIELDS", None)
+        if not fields:
+            fields = []
+            username = self._setting("USERNAME_REQUIRED", True)
+            email = self._setting("EMAIL_REQUIRED", False)
+            email2 = self._setting("SIGNUP_EMAIL_ENTER_TWICE", False)
+            password2 = self._setting(
+                "SIGNUP_PASSWORD_ENTER_TWICE",
+                self._setting("SIGNUP_PASSWORD_VERIFICATION", True),
+            )
+            if email:
+                fields.append("email*")
+            else:
+                fields.append("email")
+            if email2:
+                fields.append("email2*" if email else "email2")
+            if username:
+                fields.append("username*")
+            fields.append("password1*")
+            if password2:
+                fields.append("password2*")
+        ret = {}
+        for field in fields:
+            f, req, _ = field.partition("*")
+            ret[f] = {"required": bool(req)}
+        return ret
+
     @property
     def USERNAME_REQUIRED(self):
         """
         The user is required to enter a username when signing up
         """
-        return self._setting("USERNAME_REQUIRED", True)
+        warnings.warn(
+            "app_settings.USERNAME_REQUIRED is deprecated, use: app_settings.SIGNUP_FIELDS['username']['required']",
+            stacklevel=3,
+        )
+        username = self.SIGNUP_FIELDS.get("username")
+        return username and username["required"]
 
     @property
     def USERNAME_MIN_LENGTH(self):
@@ -366,6 +456,18 @@ class AppSettings:
                 ret = []
         return ret
 
+    @property
+    def PASSWORD_RESET_BY_CODE_ENABLED(self):
+        return self._setting("PASSWORD_RESET_BY_CODE_ENABLED", False)
+
+    @property
+    def PASSWORD_RESET_BY_CODE_MAX_ATTEMPTS(self):
+        return self._setting("PASSWORD_RESET_BY_CODE_MAX_ATTEMPTS", 3)
+
+    @property
+    def PASSWORD_RESET_BY_CODE_TIMEOUT(self):
+        return self._setting("PASSWORD_RESET_BY_CODE_TIMEOUT", 3 * 60)
+
     @property
     def PASSWORD_RESET_TOKEN_GENERATOR(self):
         from allauth.account.forms import EmailAwarePasswordResetTokenGenerator
