@@ -1,22 +1,31 @@
-from importlib import import_module
+from typing import Optional
 
 from django import forms
-from django.contrib.auth import get_user_model, password_validation
+from django.contrib.auth import (
+    REDIRECT_FIELD_NAME,
+    get_user_model,
+    password_validation,
+)
 from django.contrib.auth.tokens import PasswordResetTokenGenerator
 from django.core import exceptions, validators
+from django.template.exceptions import TemplateDoesNotExist
+from django.template.loader import render_to_string
 from django.urls import NoReverseMatch, reverse
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext, gettext_lazy as _, pgettext
 
+from allauth.account.app_settings import LoginMethod
 from allauth.account.internal import flows
+from allauth.account.internal.flows.signup import base_signup_form_class
 from allauth.account.internal.stagekit import LOGIN_SESSION_KEY
+from allauth.account.internal.textkit import compare_code
 from allauth.account.stages import EmailVerificationStage
 from allauth.core import context, ratelimit
+from allauth.core.internal.httpkit import headed_redirect_response
 from allauth.utils import get_username_max_length, set_form_field_order
 
 from . import app_settings
-from .adapter import DefaultAccountAdapter, get_adapter
-from .app_settings import AuthenticationMethod
+from .adapter import get_adapter
 from .models import EmailAddress, Login
 from .utils import (
     filter_users_by_email,
@@ -24,16 +33,13 @@ from .utils import (
     sync_user_email_addresses,
     url_str_to_user_pk,
     user_email,
-    user_pk_to_url_str,
     user_username,
 )
 
 
 class EmailAwarePasswordResetTokenGenerator(PasswordResetTokenGenerator):
     def _make_hash_value(self, user, timestamp):
-        ret = super(EmailAwarePasswordResetTokenGenerator, self)._make_hash_value(
-            user, timestamp
-        )
+        ret = super()._make_hash_value(user, timestamp)
         sync_user_email_addresses(user)
         email = user_email(user)
         emails = set([email] if email else [])
@@ -49,7 +55,7 @@ default_token_generator = app_settings.PASSWORD_RESET_TOKEN_GENERATOR()
 
 class PasswordVerificationMixin:
     def clean(self):
-        cleaned_data = super(PasswordVerificationMixin, self).clean()
+        cleaned_data = super().clean()
         password1 = cleaned_data.get("password1")
         password2 = cleaned_data.get("password2")
         if (password1 and password2) and password1 != password2:
@@ -95,8 +101,9 @@ class LoginForm(forms.Form):
 
     def __init__(self, *args, **kwargs):
         self.request = kwargs.pop("request", None)
-        super(LoginForm, self).__init__(*args, **kwargs)
-        if app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.EMAIL:
+        super().__init__(*args, **kwargs)
+        adapter = get_adapter()
+        if app_settings.LOGIN_METHODS == {LoginMethod.EMAIL}:
             login_widget = forms.EmailInput(
                 attrs={
                     "placeholder": _("Email address"),
@@ -104,7 +111,7 @@ class LoginForm(forms.Form):
                 }
             )
             login_field = forms.EmailField(label=_("Email"), widget=login_widget)
-        elif app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.USERNAME:
+        elif app_settings.LOGIN_METHODS == {LoginMethod.USERNAME}:
             login_widget = forms.TextInput(
                 attrs={"placeholder": _("Username"), "autocomplete": "username"}
             )
@@ -113,13 +120,14 @@ class LoginForm(forms.Form):
                 widget=login_widget,
                 max_length=get_username_max_length(),
             )
+        elif app_settings.LOGIN_METHODS == {LoginMethod.PHONE}:
+            login_field = adapter.phone_form_field(required=True)
         else:
-            assert (
-                app_settings.AUTHENTICATION_METHOD
-                == AuthenticationMethod.USERNAME_EMAIL
-            )  # nosec
             login_widget = forms.TextInput(
-                attrs={"placeholder": _("Username or email"), "autocomplete": "email"}
+                attrs={
+                    "placeholder": self._get_login_field_placeholder(),
+                    "autocomplete": "email",
+                }
             )
             login_field = forms.CharField(
                 label=pgettext("field label", "Login"), widget=login_widget
@@ -128,6 +136,43 @@ class LoginForm(forms.Form):
         set_form_field_order(self, ["login", "password", "remember"])
         if app_settings.SESSION_REMEMBER is not None:
             del self.fields["remember"]
+        self._setup_password_field()
+
+    def _get_login_field_placeholder(self):
+        methods = app_settings.LOGIN_METHODS
+        assert len(methods) > 1  # nosec
+        assert methods.issubset(
+            {
+                LoginMethod.USERNAME,
+                LoginMethod.EMAIL,
+                LoginMethod.PHONE,
+            }
+        )  # nosec
+        if len(methods) == 3:
+            placeholder = _("Username, email or phone")
+        elif methods == {LoginMethod.USERNAME, LoginMethod.EMAIL}:
+            placeholder = _("Username or email")
+        elif methods == {LoginMethod.USERNAME, LoginMethod.PHONE}:
+            placeholder = _("Username or phone")
+        elif methods == {LoginMethod.EMAIL, LoginMethod.PHONE}:
+            placeholder = _("Email or phone")
+        else:
+            raise ValueError(methods)
+        return placeholder
+
+    def _setup_password_field(self):
+        password_field = app_settings.SIGNUP_FIELDS.get("password1")
+        if not password_field:
+            del self.fields["password"]
+            return
+        try:
+            self.fields["password"].help_text = render_to_string(
+                "account/password_reset_help_text." + app_settings.TEMPLATE_EXTENSION
+            )
+            return
+        except TemplateDoesNotExist:
+            pass
+
         try:
             reset_url = reverse("account_reset_password")
         except NoReverseMatch:
@@ -138,41 +183,69 @@ class LoginForm(forms.Form):
                 f'<a href="{reset_url}">{forgot_txt}</a>'
             )  # nosec
 
-    def user_credentials(self):
+    def user_credentials(self) -> dict:
         """
         Provides the credentials required to authenticate the user for
         login.
         """
-        credentials = {}
         login = self.cleaned_data["login"]
-        if app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.EMAIL:
-            credentials["email"] = login
-        elif app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.USERNAME:
-            credentials["username"] = login
-        else:
-            if self._is_login_email(login):
-                credentials["email"] = login
-            credentials["username"] = login
-        credentials["password"] = self.cleaned_data["password"]
+        method = flows.login.derive_login_method(login)
+        credentials = {}
+        credentials[method] = login
+
+        # There are projects using usernames that look like email addresses,
+        # yet, really are usernames. So, if username is a login method, always
+        # give that a shot.
+        if (
+            LoginMethod.USERNAME in app_settings.LOGIN_METHODS
+            and method != LoginMethod.USERNAME
+        ):
+            credentials[LoginMethod.USERNAME] = login
+
+        password = self.cleaned_data.get("password")
+        if password:
+            credentials["password"] = password
         return credentials
 
     def clean_login(self):
         login = self.cleaned_data["login"]
         return login.strip()
 
-    def _is_login_email(self, login):
-        try:
-            validators.validate_email(login)
-            ret = True
-        except exceptions.ValidationError:
-            ret = False
-        return ret
-
     def clean(self):
-        super(LoginForm, self).clean()
+        super().clean()
         if self._errors:
             return
         credentials = self.user_credentials()
+        if "password" in credentials:
+            return self._clean_with_password(credentials)
+        return self._clean_without_password(
+            credentials.get("email"), credentials.get("phone")
+        )
+
+    def _clean_without_password(self, email: Optional[str], phone: Optional[str]):
+        """
+        If we don't have a password field, we need to replicate the request-login-code
+        behavior.
+        """
+        data = {}
+        if email:
+            data["email"] = email
+        if phone:
+            data["phone"] = phone
+        if not data:
+            self.add_error("login", get_adapter().validation_error("invalid_login"))
+        else:
+            form = RequestLoginCodeForm(data)
+            if not form.is_valid():
+                for field in ["phone", "email"]:
+                    errors = form.errors.get(field) or []  # type: ignore
+                    for error in errors:
+                        self.add_error("login", error)
+            else:
+                self.user = form._user  # type: ignore
+        return self.cleaned_data
+
+    def _clean_with_password(self, credentials: dict):
         adapter = get_adapter(self.request)
         user = adapter.authenticate(self.request, **credentials)
         if user:
@@ -180,20 +253,37 @@ class LoginForm(forms.Form):
             if flows.login.is_login_rate_limited(context.request, login):
                 raise adapter.validation_error("too_many_login_attempts")
             self._login = login
-            self.user = user
+            self.user = user  # type: ignore
         else:
-            auth_method = app_settings.AUTHENTICATION_METHOD
-            if auth_method == app_settings.AuthenticationMethod.USERNAME_EMAIL:
-                login = self.cleaned_data["login"]
-                if self._is_login_email(login):
-                    auth_method = app_settings.AuthenticationMethod.EMAIL
-                else:
-                    auth_method = app_settings.AuthenticationMethod.USERNAME
-            raise adapter.validation_error("%s_password_mismatch" % auth_method.value)
+            login_method = flows.login.derive_login_method(
+                login=self.cleaned_data["login"]
+            )
+            raise adapter.validation_error("%s_password_mismatch" % login_method.value)
         return self.cleaned_data
 
     def login(self, request, redirect_url=None):
         credentials = self.user_credentials()
+        if "password" in credentials:
+            return self._login_with_password(request, redirect_url, credentials)
+        return self._login_by_code(request, redirect_url, credentials)
+
+    def _login_by_code(self, request, redirect_url, credentials):
+        user = getattr(self, "user", None)
+        phone = credentials.get("phone")
+        email = credentials.get("email")
+        flows.login_by_code.LoginCodeVerificationProcess.initiate(
+            request=request,
+            user=user,
+            phone=phone,
+            email=email,
+        )
+        query = None
+        if redirect_url:
+            query = {}
+            query[REDIRECT_FIELD_NAME] = redirect_url
+        return headed_redirect_response("account_confirm_login_code", query=query)
+
+    def _login_with_password(self, request, redirect_url, credentials):
         login = self._login
         login.redirect_url = redirect_url
         ret = flows.login.perform_password_login(request, credentials, login)
@@ -207,55 +297,7 @@ class LoginForm(forms.Form):
         return ret
 
 
-class _DummyCustomSignupForm(forms.Form):
-    def signup(self, request, user):
-        """
-        Invoked at signup time to complete the signup of the user.
-        """
-        pass
-
-
-def _base_signup_form_class():
-    """
-    Currently, we inherit from the custom form, if any. This is all
-    not very elegant, though it serves a purpose:
-
-    - There are two signup forms: one for local accounts, and one for
-      social accounts
-    - Both share a common base (BaseSignupForm)
-
-    - Given the above, how to put in a custom signup form? Which form
-      would your custom form derive from, the local or the social one?
-    """
-    if not app_settings.SIGNUP_FORM_CLASS:
-        return _DummyCustomSignupForm
-    try:
-        fc_module, fc_classname = app_settings.SIGNUP_FORM_CLASS.rsplit(".", 1)
-    except ValueError:
-        raise exceptions.ImproperlyConfigured(
-            "%s does not point to a form class" % app_settings.SIGNUP_FORM_CLASS
-        )
-    try:
-        mod = import_module(fc_module)
-    except ImportError as e:
-        raise exceptions.ImproperlyConfigured(
-            "Error importing form class %s:" ' "%s"' % (fc_module, e)
-        )
-    try:
-        fc_class = getattr(mod, fc_classname)
-    except AttributeError:
-        raise exceptions.ImproperlyConfigured(
-            'Module "%s" does not define a' ' "%s" class' % (fc_module, fc_classname)
-        )
-    if not hasattr(fc_class, "signup"):
-        raise exceptions.ImproperlyConfigured(
-            "The custom signup form must offer"
-            " a `def signup(self, request, user)` method",
-        )
-    return fc_class
-
-
-class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
+class BaseSignupForm(base_signup_form_class()):  # type: ignore[misc]
     username = forms.CharField(
         label=_("Username"),
         min_length=app_settings.USERNAME_MIN_LENGTH,
@@ -274,12 +316,9 @@ class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
     )
 
     def __init__(self, *args, **kwargs):
-        email_required = kwargs.pop("email_required", app_settings.EMAIL_REQUIRED)
-        self.username_required = kwargs.pop(
-            "username_required", app_settings.USERNAME_REQUIRED
-        )
+        self._signup_fields = self._get_signup_fields(kwargs)
         self.account_already_exists = False
-        super(BaseSignupForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         username_field = self.fields["username"]
         username_field.max_length = get_username_max_length()
         username_field.validators.append(
@@ -287,16 +326,11 @@ class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
         )
         username_field.widget.attrs["maxlength"] = str(username_field.max_length)
 
-        default_field_order = [
-            "email",
-            "email2",  # ignored when not present
-            "username",
-            "password1",
-            "password2",  # ignored when not present
-        ]
-        if app_settings.SIGNUP_EMAIL_ENTER_TWICE:
+        email2 = self._signup_fields.get("email2")
+        if email2:
             self.fields["email2"] = forms.EmailField(
                 label=_("Email (again)"),
+                required=email2["required"],
                 widget=forms.TextInput(
                     attrs={
                         "type": "email",
@@ -304,31 +338,68 @@ class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
                     }
                 ),
             )
-        if email_required:
-            self.fields["email"].label = gettext("Email")
-            self.fields["email"].required = True
+        email = self._signup_fields.get("email")
+        if email:
+            if email["required"]:
+                self.fields["email"].label = gettext("Email")
+                self.fields["email"].required = True
+            else:
+                self.fields["email"].label = gettext("Email (optional)")
+                self.fields["email"].required = False
+                self.fields["email"].widget.is_required = False
         else:
-            self.fields["email"].label = gettext("Email (optional)")
-            self.fields["email"].required = False
-            self.fields["email"].widget.is_required = False
-            if self.username_required:
-                default_field_order = [
-                    "username",
-                    "email",
-                    "email2",  # ignored when not present
-                    "password1",
-                    "password2",  # ignored when not present
-                ]
+            del self.fields["email"]
 
-        if not self.username_required:
+        username = self._signup_fields.get("username")
+        if username:
+            if username["required"]:
+                self.fields["username"].label = gettext("Username")
+                self.fields["username"].required = True
+            else:
+                self.fields["username"].label = gettext("Username (optional)")
+                self.fields["username"].required = False
+                self.fields["username"].widget.is_required = False
+        else:
             del self.fields["username"]
 
+        phone = self._signup_fields.get("phone")
+        self._has_phone_field = bool(phone)
+        if phone:
+            adapter = get_adapter()
+            self.fields["phone"] = adapter.phone_form_field(
+                label=_("Phone"), required=phone["required"]
+            )
+
+        default_field_order = list(self._signup_fields.keys())
         set_form_field_order(
             self, getattr(self, "field_order", None) or default_field_order
         )
 
+    def _get_signup_fields(self, kwargs):
+        signup_fields = app_settings.SIGNUP_FIELDS
+        if "email_required" in kwargs:
+            email = signup_fields.get("email")
+            if not email:
+                raise exceptions.ImproperlyConfigured(
+                    "email required but not listed as a field"
+                )
+            email["required"] = kwargs.pop("email_required")
+            email2 = signup_fields.get("email2")
+            if email2:
+                email2["required"] = email["required"]
+        if "username_required" in kwargs:
+            username = signup_fields.get("username")
+            if not username:
+                raise exceptions.ImproperlyConfigured(
+                    "username required but not listed as a field"
+                )
+            username["required"] = kwargs.pop("username_required")
+        return signup_fields
+
     def clean_username(self):
         value = self.cleaned_data["username"]
+        if not value and not self._signup_fields["username"]["required"]:
+            return value
         value = get_adapter().clean_username(value)
         # Note regarding preventing enumeration: if the username is already
         # taken, but the email address is not, we would still leak information
@@ -363,7 +434,7 @@ class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
 
     def clean(self):
         cleaned_data = super(BaseSignupForm, self).clean()
-        if app_settings.SIGNUP_EMAIL_ENTER_TWICE:
+        if "email2" in self._signup_fields:
             email = cleaned_data.get("email")
             email2 = cleaned_data.get("email2")
             if (email and email2) and email != email2:
@@ -398,6 +469,10 @@ class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
         adapter = get_adapter()
         user = adapter.new_user(request)
         adapter.save_user(request, user, self)
+        if self._has_phone_field:
+            phone = self.cleaned_data.get("phone")
+            if phone:
+                adapter.set_phone(user, phone, False)
         self.custom_signup(request, user)
         # TODO: Move into adapter `save_user` ?
         setup_user_email(request, user, [EmailAddress(email=email)] if email else [])
@@ -407,16 +482,20 @@ class BaseSignupForm(_base_signup_form_class()):  # type: ignore[misc]
 class SignupForm(BaseSignupForm):
     def __init__(self, *args, **kwargs):
         self.by_passkey = kwargs.pop("by_passkey", False)
-        super(SignupForm, self).__init__(*args, **kwargs)
-        if not self.by_passkey:
+        super().__init__(*args, **kwargs)
+        password1_field = self._signup_fields.get("password1")
+        if not self.by_passkey and password1_field:
             self.fields["password1"] = PasswordField(
                 label=_("Password"),
                 autocomplete="new-password",
                 help_text=password_validation.password_validators_help_text_html(),
+                required=password1_field["required"],
             )
-            if app_settings.SIGNUP_PASSWORD_ENTER_TWICE:
+            if "password2" in self._signup_fields:
                 self.fields["password2"] = PasswordField(
-                    label=_("Password (again)"), autocomplete="new-password"
+                    label=_("Password (again)"),
+                    autocomplete="new-password",
+                    required=password1_field["required"],
                 )
 
         if hasattr(self, "field_order"):
@@ -473,7 +552,7 @@ class SignupForm(BaseSignupForm):
                 self.add_error("password1", e)
 
         if (
-            app_settings.SIGNUP_PASSWORD_ENTER_TWICE
+            "password2" in self._signup_fields
             and "password1" in self.cleaned_data
             and "password2" in self.cleaned_data
         ):
@@ -604,33 +683,17 @@ class ResetPasswordForm(forms.Form):
 
     def save(self, request, **kwargs) -> str:
         email = self.cleaned_data["email"]
-        if not self.users:
-            flows.signup.send_unknown_account_mail(request, email)
-            return email
-
-        adapter: DefaultAccountAdapter = get_adapter()
-        token_generator = kwargs.get("token_generator", default_token_generator)
-        for user in self.users:
-            temp_key = token_generator.make_token(user)
-
-            # send the password reset email
-            uid = user_pk_to_url_str(user)
-            # We intentionally pass an opaque `key` on the interface here, and
-            # not implementation details such as a separate `uidb36` and
-            # `key. Ideally, this should have done on `urls` level as well.
-            key = f"{uid}-{temp_key}"
-            url = adapter.get_reset_password_from_key_url(key)
-            context = {
-                "user": user,
-                "password_reset_url": url,
-                "uid": uid,
-                "key": temp_key,
-                "request": request,
-            }
-
-            if app_settings.AUTHENTICATION_METHOD != AuthenticationMethod.EMAIL:
-                context["username"] = user_username(user)
-            adapter.send_password_reset_mail(user, email, context)
+        if app_settings.PASSWORD_RESET_BY_CODE_ENABLED:
+            flows.password_reset_by_code.PasswordResetVerificationProcess.initiate(
+                request=request,
+                user=(self.users[0] if self.users else None),
+                email=email,
+            )
+        else:
+            token_generator = kwargs.get("token_generator", default_token_generator)
+            flows.password_reset.request_password_reset(
+                request, email, self.users, token_generator
+            )
         return email
 
 
@@ -641,7 +704,7 @@ class ResetPasswordKeyForm(PasswordVerificationMixin, forms.Form):
     def __init__(self, *args, **kwargs):
         self.user = kwargs.pop("user", None)
         self.temp_key = kwargs.pop("temp_key", None)
-        super(ResetPasswordKeyForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.fields["password1"].user = self.user
 
     def save(self):
@@ -705,18 +768,55 @@ class RequestLoginCodeForm(forms.Form):
         )
     )
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._has_email = LoginMethod.EMAIL in app_settings.LOGIN_METHODS
+        self._has_phone = LoginMethod.PHONE in app_settings.LOGIN_METHODS
+        if self._has_phone:
+            adapter = get_adapter()
+            self.fields["phone"] = adapter.phone_form_field(
+                required=not self._has_email
+            )
+            self.fields["email"].required = False
+        # Inconsistent, but kept for backwards compatibility: even if email is not a login
+        # method the email field is added. May be used when login is by username.
+        if self._has_phone and not self._has_email:
+            self.fields.pop("email")
+
+    def clean(self):
+        cleaned_data = super().clean()
+        adapter = get_adapter()
+        phone = cleaned_data.get("phone")
+        email = cleaned_data.get("email")
+        if email and phone:
+            raise adapter.validation_error("select_only_one")
+        return cleaned_data
+
+    def clean_phone(self):
+        adapter = get_adapter()
+        phone = self.cleaned_data["phone"]
+        if phone:
+            self._user = adapter.get_user_by_phone(phone)
+            if not self._user and not app_settings.PREVENT_ENUMERATION:
+                raise adapter.validation_error("unknown_phone")
+            if not ratelimit.consume(
+                context.request, action="request_login_code", key=phone.lower()
+            ):
+                raise adapter.validation_error("too_many_login_attempts")
+        return phone
+
     def clean_email(self):
         adapter = get_adapter()
         email = self.cleaned_data["email"]
+        users = filter_users_by_email(email, is_active=True, prefer_verified=True)
         if not app_settings.PREVENT_ENUMERATION:
-            users = filter_users_by_email(email, is_active=True, prefer_verified=True)
             if not users:
                 raise adapter.validation_error("unknown_email")
-
         if not ratelimit.consume(
             context.request, action="request_login_code", key=email.lower()
         ):
             raise adapter.validation_error("too_many_login_attempts")
+        self._user = users[0] if users else None
         return email
 
 
@@ -734,7 +834,7 @@ class BaseConfirmCodeForm(forms.Form):
 
     def clean_code(self):
         code = self.cleaned_data.get("code")
-        if not flows.login_by_code.compare_code(actual=code, expected=self.code):
+        if not compare_code(actual=code, expected=self.code):
             raise get_adapter().validation_error("incorrect_code")
         return code
 
@@ -745,3 +845,25 @@ class ConfirmLoginCodeForm(BaseConfirmCodeForm):
 
 class ConfirmEmailVerificationCodeForm(BaseConfirmCodeForm):
     pass
+
+
+class ConfirmPasswordResetCodeForm(BaseConfirmCodeForm):
+    pass
+
+
+class VerifyPhoneForm(BaseConfirmCodeForm):
+    pass
+
+
+class ChangePhoneForm(forms.Form):
+    def __init__(self, *args, **kwargs):
+        self.phone = kwargs.pop("phone")
+        super().__init__(*args, **kwargs)
+        adapter = get_adapter()
+        self.fields["phone"] = adapter.phone_form_field(required=True)
+
+    def clean_phone(self):
+        phone = self.cleaned_data["phone"]
+        if phone == self.phone:
+            raise get_adapter().validation_error("same_as_current")
+        return phone
