@@ -1,7 +1,4 @@
-from typing import Optional
-
-from django.http import HttpResponseRedirect
-from django.urls import reverse
+from typing import List, Optional
 
 from allauth.account import app_settings
 from allauth.account.adapter import get_adapter
@@ -40,7 +37,7 @@ class LoginStage:
         from allauth.account.internal.stagekit import clear_login
 
         clear_login(self.request)
-        return HttpResponseRedirect(reverse("account_login"))
+        return headed_redirect_response("account_login")
 
     def is_resumable(self, request):
         return True
@@ -88,7 +85,13 @@ class LoginStageController:
             break
         return ret
 
-    def get_stages(self):
+    def get_stage(self, key: str) -> Optional[LoginStage]:
+        try:
+            return next(iter(stage for stage in self.get_stages() if stage.key == key))
+        except StopIteration:
+            return None
+
+    def get_stages(self) -> List[LoginStage]:
         stages = []
         adapter = get_adapter(self.request)
         paths = adapter.get_login_stages()
@@ -161,20 +164,55 @@ class LoginByCodeStage(LoginStage):
     def handle(self):
         from allauth.account.internal.flows import login_by_code
 
-        user, data = login_by_code.get_pending_login(
-            self.request, self.login, peek=True
-        )
+        did_initiate_process = bool(self.state)
         login_by_code_required = get_adapter().is_login_by_code_required(self.login)
-        if data is None and not login_by_code_required:
-            # No pending login, just continue.
+        if not did_initiate_process and not login_by_code_required:
+            # Didn't initiate login by code process, but not required, so continue.
             return None, True
-        elif data is None and login_by_code_required:
+        elif not did_initiate_process and login_by_code_required:
             email = EmailAddress.objects.get_primary_email(self.login.user)
-            if not email:
+            phone = None
+            phone_field = app_settings.SIGNUP_FIELDS.get("phone")
+            if not email and phone_field:
+                phone_verified = get_adapter().get_phone(self.login.user)
+                if phone_verified:
+                    phone = phone_verified[0]
+            if not email and not phone:
                 # No way of contacting the user.. cannot meet the
                 # requirements. Abort.
                 return headed_redirect_response("account_login"), False
-            login_by_code.request_login_code(self.request, email, login=self.login)
+            login_by_code.LoginCodeVerificationProcess.initiate(
+                request=self.request,
+                user=self.login.user,
+                phone=phone,
+                email=email,
+                stage=self,
+            )
 
         response = headed_redirect_response("account_confirm_login_code")
         return response, True
+
+
+class PhoneVerificationStage(LoginStage):
+    key = "verify_phone"
+    urlname = "account_verify_phone"
+
+    def handle(self):
+        from allauth.account.internal.flows import phone_verification
+
+        phone_field = app_settings.SIGNUP_FIELDS.get("phone")
+        if not phone_field:
+            return None, True
+        adapter = get_adapter()
+
+        phone_verified = adapter.get_phone(self.login.user)
+        if phone_verified is None:
+            return None, (not phone_field["required"])
+        phone, verified = phone_verified
+        if verified or not app_settings.PHONE_VERIFICATION_ENABLED:
+            return None, True
+        phone_verification.PhoneVerificationStageProcess.initiate(
+            stage=self, phone=phone
+        )
+        response = headed_redirect_response("account_verify_phone")
+        return response, True
